/* license: AGPLv3 or later */
/* copyright 2024 zamfofex */

#include <stdlib.h>

#include <mimimi-chapters.h>

#include "ground.c"
#include "background.c"

static void mimimi_chapter_I_background(struct mimimi_chapter_I *chapter)
{
	static unsigned char colors[][5] = {{0}, {6, 0x3D, 0xC4}, {0x7D, 0x4D, 0x70, 8, 14}};
	
	int i;
	struct mimimi_area *area;
	struct mimimi_image *image;
	
	for (i = 1 ; i < chapter->stage.area_count ; i++)
	{
		area = chapter->stage.areas + i;
		area->background_count = 1;
		area->backgrounds = malloc(sizeof *area->backgrounds);
		if (area->backgrounds == NULL) exit(1);
		area->backgrounds[0].texture = mimimi_background(chapter->engine, area->ground, colors[i]);
		area->backgrounds[0].x = 0;
		area->backgrounds[0].y = 0;
		area->backgrounds[0].z = 0;
	}
	
	image = malloc(sizeof *image);
	if (image == NULL) exit(1);
	image->width = mimimi_chapter_I_background_width;
	image->height = mimimi_chapter_I_background_height;
	image->colors = malloc(image->width * image->height);
	if (image->colors == NULL) exit(1);
	mimimi_unpack_bits(image->width * image->height, image->colors, mimimi_chapter_I_background_packed);
	chapter->stage.areas[0].background_count = 1;
	chapter->stage.areas[0].backgrounds = malloc(sizeof *chapter->stage.areas[0].backgrounds);
	if (chapter->stage.areas[0].backgrounds == NULL) exit(1);
	chapter->stage.areas[0].backgrounds[0].texture = (*chapter->engine->texture)(chapter->engine->data, image);
	chapter->stage.areas[0].backgrounds[0].x = -88;
	chapter->stage.areas[0].backgrounds[0].y = 0;
	chapter->stage.areas[0].backgrounds[0].z = 0;
	
	image = malloc(sizeof *image);
	if (image == NULL) exit(1);
	image->width = mimimi_chapter_I_foreground_width;
	image->height = mimimi_chapter_I_foreground_height;
	image->colors = malloc(image->width * image->height);
	if (image->colors == NULL) exit(1);
	mimimi_unpack_bits(image->width * image->height, image->colors, mimimi_chapter_I_foreground_packed);
	chapter->stage.areas[0].foreground_count = 1;
	chapter->stage.areas[0].foregrounds = malloc(sizeof *chapter->stage.areas[0].foregrounds);
	if (chapter->stage.areas[0].foregrounds == NULL) exit(1);
	chapter->stage.areas[0].foregrounds[0].texture = (*chapter->engine->texture)(chapter->engine->data, image);
	chapter->stage.areas[0].foregrounds[0].x = 0;
	chapter->stage.areas[0].foregrounds[0].y = 0;
	chapter->stage.areas[0].foregrounds[0].z = 0;
}

void mimimi_chapter_I_tick(void *data, unsigned char left, unsigned char right)
{
	struct mimimi_chapter_I *chapter;
	struct mimimi_sprite *player;
	
	chapter = data;
	player = chapter->stage.sprites;
	
	mimimi_history_tick(&chapter->history, left, right);
	mimimi_controls_tick(&chapter->stage.sprites[0].walk, &chapter->history, &mimimi_jump, chapter->stage.sprites);
	mimimi_stage_tick(&chapter->stage, chapter->engine);
	
	if (chapter->progression == 0)
	{
		if (player->position.x < -9216)
			chapter->progression++;
	}
	if (chapter->progression == 1)
	{
		if (mimimi_toast_dialogue_bottom_tick(&chapter->dialogue0) != 0)
			chapter->progression++;
	}
	if (chapter->progression == 2)
	{
		if (chapter->stage.platform_sets[0].angular_platforms[0].physics.other.airborne == 0)
			chapter->progression++;
	}
	if (chapter->progression == 3)
	{
		if (mimimi_toast_dialogue_bottom_tick(&chapter->dialogue1) != 0)
			chapter->progression++;
	}
	if (chapter->progression == 4)
	{
		if (player->position.y <= -3712 && player->physics.airborne == 0)
			chapter->progression++;
	}
	if (chapter->progression == 5)
	{
		if (mimimi_toast_dialogue_bottom_tick(&chapter->dialogue) != 0)
			chapter->progression++;
	}
}

void *mimimi_chapter_I(struct mimimi_engine *engine)
{
	static unsigned char colors[] = {0, 0x6B};
	static struct mimimi_ground ground = {4, 3, {0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0}};
	static struct mimimi_ground *grounds[] = {&mimimi_chapter_I_hideout, &mimimi_chapter_I_stairs, &mimimi_chapter_I_platforming};
	static struct mimimi_dialogue_paragraph paragraphs[] =
	{
		{NULL, "Mirai", "There seems to be nothing for me to do. Hmmm... Maybe I can find something to do later."},
		{NULL, "Ryouna", "Hear that? If you come back later, then maybe you can find something to do!"},
		{NULL, "Ryoubi", "Who are you talking to? Are you stupid or something?"},
		{NULL, "Ryouna", "Aaah! Ryoubi, I really like when you yell insults at me!"},
		{NULL, "Mirai", "Wha-? What are you two doing here?"},
		{NULL, "Haruka", "Hehe, don't worry about it!"},
	};
	static struct mimimi_dialogue_paragraph paragraphs0[] =
	{
		{NULL, "Mirai", "Hmm, I wonder what's on top of those platforms..."},
		{NULL, "Mirai", "Oh, it seems like I need to be quick if I want to get to the top."},
	};
	
	struct mimimi_dialogue dialogue;
	struct mimimi_chapter_I *chapter;
	struct mimimi_display *display;
	struct mimimi_platform_set *platforms;
	void *texture;
	
	paragraphs0[0].model = mimimi_mirai_flash;
	paragraphs0[1].model = mimimi_mirai_flash;
	paragraphs[0].model = mimimi_mirai_flash;
	paragraphs[1].model = mimimi_ryouna_flash;
	paragraphs[2].model = mimimi_ryoubi_flash;
	paragraphs[3].model = mimimi_ryouna_flash;
	paragraphs[4].model = mimimi_mirai_flash;
	paragraphs[5].model = mimimi_haruka_flash;
	
	chapter = malloc(sizeof *chapter);
	if (chapter == NULL) exit(1);
	chapter->engine = engine;
	
	display = malloc(sizeof *display);
	if (display == NULL) exit(1);
	
	mimimi_stage(&chapter->stage, grounds, sizeof grounds / sizeof *grounds);
	
	chapter->progression = 0;
	chapter->progression_counter = 0;
	
	dialogue.count = 1;
	dialogue.paragraphs = paragraphs0;
	mimimi_toast_dialogue(&chapter->dialogue0, &dialogue, engine);
	
	dialogue.count = 1;
	dialogue.paragraphs = paragraphs0 + 1;
	mimimi_toast_dialogue(&chapter->dialogue1, &dialogue, engine);
	
	dialogue.count = sizeof paragraphs / sizeof *paragraphs;
	dialogue.paragraphs = paragraphs;
	mimimi_toast_dialogue(&chapter->dialogue, &dialogue, engine);
	
	mimimi_display(display, mimimi_mirai_flash, engine);
	mimimi_spawn(&chapter->stage, display, 0, 0, 80, 250);
	
	texture = mimimi_background(chapter->engine, &ground, colors);
	platforms = mimimi_add_platform_set(&chapter->stage);
	platforms->strength = 64;
	platforms->max = 0x1300;
	mimimi_spawn_angular_platform(&chapter->stage, platforms, &ground, texture, -0x2600, -2048, -80, 256, 1024);
	mimimi_spawn_angular_platform(&chapter->stage, platforms, &ground, texture, -0x2600, -2048, -80, 256, 2048);
	mimimi_spawn_angular_platform(&chapter->stage, platforms, &ground, texture, -0x2600, -2048, -80, 256, 3072);
	mimimi_spawn_angular_platform(&chapter->stage, platforms, &ground, texture, -0x2600, -2048, -80, 256, 4096);
	
	chapter->history.left = 0;
	chapter->history.right = 0;
	
	mimimi_chapter_I_background(chapter);
	
	return chapter;
}
