/* license: AGPLv3 or later */
/* copyright 2024 zamfofex */

#include <stdlib.h>

#include <mimimi-chapters.h>

struct mimimi_loader_chapter_info
{
	char *name, *description;
	void *(*start)(struct mimimi_engine *engine);
	void (*tick)(void *chapter, unsigned char left, unsigned char right);
};

static struct mimimi_loader_chapter_info mimimi_loader_chapters[] =
{
	{
		"test chapter", "This is a chapter to test the foundations of the game and its engine. It features the characters Mango and Pepper!",
		&mimimi_test, &mimimi_test_tick,
	},
	{
		"chapter I", "This is the first chapter of the game! It is currently still a work-in-progress, but strives to eventually come to tell a story.",
		&mimimi_chapter_I, &mimimi_chapter_I_tick,
	},
};

static void mimimi_loader_fill(struct mimimi_image *image, unsigned char color)
{
	int i;
	for (i = 0 ; i < image->width * image->height ; i++)
		image->colors[i] = color;
}

void mimimi_loader_tick(void *data, unsigned char left, unsigned char right)
{
	static int count = sizeof mimimi_loader_chapters / sizeof *mimimi_loader_chapters;
	
	struct mimimi_loader *loader;
	int i;
	int x, y;
	int width, height;
	
	loader = data;
	
	if (loader->started == 2)
	{
		(*mimimi_loader_chapters[loader->index].tick)(loader->chapter, left, right);
		return;
	}
	
	if (left != 0 && right != 0)
		loader->started = 1;
	
	if (loader->started == 1)
	{
		if (left == 0 && right == 0)
		{
			(*loader->engine->invalidate)(loader->engine->data, loader->background);
			for (i = 0 ; i < count ; i++)
				(*loader->engine->invalidate)(loader->engine->data, loader->textures[i]);
			
			loader->started = 2;
			loader->chapter = (*mimimi_loader_chapters[loader->index].start)(loader->engine);
			(*mimimi_loader_chapters[loader->index].tick)(loader->chapter, left, right);
			
			return;
		}
	}
	else
	{
		if (left == 0)
		if (loader->previous_direction == -1)
		if (loader->index > 0)
			loader->index--;
		
		if (right == 0)
		if (loader->previous_direction == 1)
		if (loader->index < count - 1)
			loader->index++;
		
		loader->previous_direction = 0;
		if (left != 0) loader->previous_direction = -1;
		if (right != 0) loader->previous_direction = 1;
	}
	
	width = loader->images[0].width;
	height = loader->images[0].height;
	
	loader->x *= 3;
	loader->x += (width + 8) * loader->index;
	loader->x /= 4;
	
	(*loader->engine->stamp)(loader->engine->data, 0, 0, loader->background);
	for (i = 0 ; i < count ; i++)
	{
		x = loader->engine->size.width / 2 - width / 2;
		y = loader->engine->size.height / 2 - height / 2;
		x += (width + 8) * i - loader->x;
		(*loader->engine->stamp)(loader->engine->data, x, y, loader->textures[i]);
	}
}

void *mimimi_loader(struct mimimi_engine *engine)
{
	static int count = sizeof mimimi_loader_chapters / sizeof *mimimi_loader_chapters;
	static unsigned char background_colors[512 * 256];
	static char title[] = "select chapter";
	static char note[] = "press [left] and [right] at the same time to select";
	
	struct mimimi_loader *loader;
	int i;
	unsigned char *colors;
	struct mimimi_image *image;
	struct mimimi_loader_chapter_info *info;
	int y;
	int width;
	
	loader = malloc(sizeof *loader);
	if (loader == NULL) exit(1);
	
	loader->index = 0;
	loader->started = 0;
	loader->engine = engine;
	loader->previous_direction = 0;
	loader->x = 0;
	
	loader->background_image.colors = background_colors;
	loader->background_image.width = 512;
	loader->background_image.height = 256;
	mimimi_loader_fill(&loader->background_image, 0x16);
	
	width = mimimi_measure_paragraph(mimimi_font, title);
	mimimi_draw_paragraph(mimimi_font, title, &loader->background_image, loader->background_image.width / 2 - width / 2, 32, 0xD);
	
	width = mimimi_measure_paragraph(mimimi_font, note);
	mimimi_draw_paragraph(mimimi_font, note, &loader->background_image, loader->background_image.width / 2 - width / 2, loader->background_image.height - 32 - 16, 0xD);
	
	loader->background = (*loader->engine->texture)(loader->engine->data, &loader->background_image);
	
	colors = loader->colors;
	
	for (i = 0 ; i < count ; i++)
	{
		info = mimimi_loader_chapters + i;
		image = loader->images + i;
		image->width = 192;
		image->height = 96;
		image->colors = colors;
		colors += image->width * image->height;
		mimimi_loader_fill(image, 0xD);
		y = mimimi_draw_text(mimimi_font, info->name, 172, 18, image, 8, 2, 4);
		mimimi_draw_text(mimimi_font, info->description, 172, 18, image, 8, y + 4, 4);
		loader->textures[i] = (*engine->texture)(engine->data, image);
	}
	
	return loader;
}
