/* license: AGPLv3 or later */
/* copyright 2024 zamfofex */

#include <stdlib.h>

#include <mimimi-chapters.h>

#include "ground.c"

void mimimi_test_tick(void *data, unsigned char left, unsigned char right)
{
	struct mimimi_test *chapter;
	chapter = data;
	mimimi_history_tick(&chapter->history, left, right);
	mimimi_controls_tick(&chapter->stage.sprites[0].walk, &chapter->history, &mimimi_jump, chapter->stage.sprites);
	mimimi_stage_tick(&chapter->stage, chapter->engine);
}

static void mimimi_test_background(struct mimimi_test *chapter)
{
	static unsigned char colors[] = {0x0D, 0x05, 0x05};
	
	struct mimimi_area *area;
	
	area = chapter->stage.areas;
	area->background_count = 1;
	area->backgrounds = malloc(sizeof *area->backgrounds);
	if (area->backgrounds == NULL) exit(1);
	area->backgrounds[0].x = 0;
	area->backgrounds[0].y = 0;
	area->backgrounds[0].z = 0;
	area->backgrounds[0].texture = mimimi_background(chapter->engine, area->ground, colors);
}

void *mimimi_test(struct mimimi_engine *engine)
{
	static struct mimimi_ground *grounds[] = {&mimimi_test_ground};
	static struct mimimi_display mango_display, pepper_display;
	struct mimimi_test *chapter;
	mimimi_display(&mango_display, mimimi_mango, engine);
	mimimi_display(&pepper_display, mimimi_pepper, engine);
	chapter = malloc(sizeof *chapter);
	if (chapter == NULL) exit(1);
	chapter->engine = engine;
	mimimi_stage(&chapter->stage, grounds, 1);
	mimimi_test_background(chapter);
	mimimi_spawn(&chapter->stage, &mango_display, 0, 0, 80, 250);
	mimimi_spawn(&chapter->stage, &pepper_display, -256, 0, 80, 250);
	chapter->history.left = 0;
	chapter->history.right = 0;
	return chapter;
}
