/* license: AGPLv3 or later */
/* copyright 2024 zamfofex */

#include <stdlib.h>

#include <mimimi.h>

void mimimi_conveyor_tick(struct mimimi_conveyor *conveyor)
{
	struct mimimi_sprite *sprite;
	struct mimimi_ground *ground;
	unsigned char was_airborne;
	int x1, x2, y;
	
	sprite = conveyor->sprite;
	ground = conveyor->ground;
	was_airborne = conveyor->was_airborne;
	
	conveyor->was_airborne = sprite->physics.airborne;
	
	if (sprite->physics.airborne != 0)
	{
		if (was_airborne == 0)
			sprite->physics.dx += conveyor->speed;
		return;
	}
	
	x1 = (sprite->position.x - sprite->physics.width / 2) / 128;
	x2 = (sprite->position.x + sprite->physics.width / 2) / 128;
	y = (sprite->position.y + 64) / 128;
	
	if (mimimi_ground_tile(ground, x1, y) != 0 || mimimi_ground_tile(ground, x2, y) != 0)
		sprite->position.x += conveyor->speed;
}

void mimimi_positioned_physics_tick(struct mimimi_positioned_physics *physics, struct mimimi_ground *ground)
{
	int dx, dy;
	unsigned char was_airborne;
	struct mimimi_position other_position;
	unsigned char airborne;
	
	dx = physics->transform.x - physics->previous.x;
	dy = physics->transform.y - physics->previous.y;
	
	physics->previous = physics->transform;
	
	was_airborne = physics->was_airborne;
	physics->was_airborne = physics->other.airborne;
	
	airborne = physics->physics->airborne;
	
	physics->other.dx = physics->physics->dx;
	physics->other.dy = physics->physics->dy;
	physics->other.width = physics->physics->width;
	physics->other.height = physics->physics->height;
	
	other_position = *physics->position;
	other_position.x -= physics->transform.x;
	other_position.y -= physics->transform.y;
	
	if (physics->other.airborne == 0)
	{
		other_position.x += dx;
		other_position.y += dy;
	}
	
	if (physics->physics->airborne != 0) physics->other.airborne = 1;
	mimimi_collision_physics_tick(&physics->other, &other_position, ground);
	if (physics->other.airborne == 0) physics->physics->airborne = 0;
	
	if (airborne == 0) return;
	
	other_position.x += physics->transform.x;
	other_position.y += physics->transform.y;
	
	*physics->position = other_position;
	
	physics->physics->dx = physics->other.dx;
	physics->physics->dy = physics->other.dy;
	
	if (physics->other.airborne != 0 && was_airborne == 0)
	{
		/* physics->physics->dx += dx; */
		/* physics->physics->dy += dy; */
	}
}

void mimimi_positioned_physics(struct mimimi_positioned_physics *physics, struct mimimi_physics *other, struct mimimi_position *position)
{
	mimimi_physics(&physics->other, 0, 0);
	physics->physics = other;
	physics->position = position;
	physics->transform.x = 0;
	physics->transform.y = 0;
	physics->previous = *position;
	physics->was_airborne = other->airborne;
}

void mimimi_trampoline_tick(struct mimimi_trampoline *trampoline)
{
	struct mimimi_sprite *sprite;
	struct mimimi_ground *ground;
	int dy;
	int x1, x2, y;
	
	sprite = trampoline->sprite;
	ground = trampoline->ground;
	
	dy = trampoline->dy;
	trampoline->dy = 0;
	
	if (sprite->physics.airborne != 0)
	{
		trampoline->dy = sprite->physics.dy;
		return;
	}
	
	if (dy == 0) return;
	
	x1 = (sprite->position.x - sprite->physics.width / 2) / 128;
	x2 = (sprite->position.x + sprite->physics.width / 2) / 128;
	y = (sprite->position.y + 64) / 128;
	
	if (mimimi_ground_tile(ground, x1, y) != 0) return;
	if (mimimi_ground_tile(ground, x2, y) != 0) return;
	
	sprite->physics.airborne = 1;
	sprite->physics.dy -= dy * trampoline->restitution / 256;
}

void mimimi_platform_display_tick(struct mimimi_platform *platform, struct mimimi_position *camera, int x, int y, struct mimimi_engine *engine)
{
	mimimi_camera_stamp(engine, camera, platform->physics.transform.x - x, platform->physics.transform.y - y, 0, platform->texture);
}

struct mimimi_platform_set *mimimi_add_platform_set(struct mimimi_stage *stage)
{
	int i;
	struct mimimi_platform_set *platforms;
	
	if (stage->platform_set_count == 0)
		stage->platform_sets = NULL;
	
	i = stage->platform_set_count++;
	stage->platform_sets = realloc(stage->platform_sets, sizeof *stage->platform_sets * stage->platform_set_count);
	
	platforms = stage->platform_sets + i;
	
	platforms->angular_platform_count = 0;
	platforms->linear_platform_count = 0;
	platforms->momentum.x = 0;
	platforms->momentum.dx = 0;
	platforms->balance = 0;
	platforms->strength = 0;
	platforms->linear_weight = 0;
	platforms->angular_weight = 1024;
	platforms->min = -0x10000000;
	platforms->max = 0x10000000;
	
	return platforms;
}

void mimimi_spawn_linear_platform(struct mimimi_stage *stage, struct mimimi_platform_set *platforms, struct mimimi_ground *ground, void *texture, int x, int y, int scale)
{
	int i;
	
	if (platforms->linear_platform_count == 0)
	{
		platforms->linear_platforms = NULL;
		platforms->linear_motions = NULL;
	}
	
	i = platforms->linear_platform_count++;
	platforms->linear_platforms = realloc(platforms->linear_platforms, sizeof *platforms->linear_platforms * platforms->linear_platform_count);
	platforms->linear_motions = realloc(platforms->linear_motions, sizeof *platforms->linear_motions * platforms->linear_platform_count);
	
	mimimi_positioned_physics(&platforms->linear_platforms[i].physics, &stage->sprites[0].physics, &stage->sprites[0].position);
	platforms->linear_platforms[i].ground = ground;
	platforms->linear_platforms[i].texture = texture;
	
	platforms->linear_platforms[i].physics.transform.x = x;
	mimimi_linear_motion(platforms->linear_motions + i, &platforms->linear_platforms[i].physics.transform.y, y, scale);
	mimimi_linear_motion_apply(platforms->linear_motions + i, &platforms->momentum);
	
	for (i = 0 ; i < platforms->linear_platform_count ; i++)
		platforms->linear_motions[i].value = &platforms->linear_platforms[i].physics.transform.y;
}

void mimimi_spawn_angular_platform(struct mimimi_stage *stage, struct mimimi_platform_set *platforms, struct mimimi_ground *ground, void *texture, int x, int y, int a, int scale, int height)
{
	int i;
	
	if (platforms->angular_platform_count == 0)
	{
		platforms->angular_platforms = NULL;
		platforms->angular_motions = NULL;
	}
	
	i = platforms->angular_platform_count++;
	platforms->angular_platforms = realloc(platforms->angular_platforms, sizeof *platforms->angular_platforms * platforms->angular_platform_count);
	platforms->angular_motions = realloc(platforms->angular_motions, sizeof *platforms->angular_motions * platforms->angular_platform_count);
	
	mimimi_positioned_physics(&platforms->angular_platforms[i].physics, &stage->sprites[0].physics, &stage->sprites[0].position);
	platforms->angular_platforms[i].ground = ground;
	platforms->angular_platforms[i].texture = texture;
	
	mimimi_angular_motion(platforms->angular_motions + i, &platforms->angular_platforms[i].physics.transform, x, y, a, scale, height);
	mimimi_angular_motion_apply(platforms->angular_motions + i, &platforms->momentum);
	
	for (i = 0 ; i < platforms->angular_platform_count ; i++)
		platforms->angular_motions[i].value = &platforms->angular_platforms[i].physics.transform;
}

static void mimimi_platforms_propagate(struct mimimi_platform_set *platforms)
{
	int i;
	for (i = 0 ; i < platforms->linear_platform_count ; i++)
		if (platforms->linear_platforms[i].physics.other.airborne == 0)
			mimimi_linear_motion_propagate(platforms->linear_motions + i, &platforms->momentum);
	for (i = 0 ; i < platforms->angular_platform_count ; i++)
		if (platforms->angular_platforms[i].physics.other.airborne == 0)
			mimimi_angular_motion_propagate(platforms->angular_motions + i, &platforms->momentum);
}

static void mimimi_platforms_apply(struct mimimi_platform_set *platforms)
{
	int i;
	for (i = 0 ; i < platforms->linear_platform_count ; i++)
		mimimi_linear_motion_apply(platforms->linear_motions + i, &platforms->momentum);
	for (i = 0 ; i < platforms->angular_platform_count ; i++)
		mimimi_angular_motion_apply(platforms->angular_motions + i, &platforms->momentum);
}

void mimimi_platform_set_tick(struct mimimi_platform_set *platforms)
{
	int i;
	int diff;
	
	for (i = 0 ; i < platforms->linear_platform_count ; i++)
		if (platforms->linear_platforms[i].physics.other.airborne == 0)
			platforms->linear_platforms[i].physics.transform.y += platforms->linear_weight;
	
	for (i = 0 ; i < platforms->angular_platform_count ; i++)
		if (platforms->angular_platforms[i].physics.other.airborne == 0)
			platforms->angular_platforms[i].physics.transform.y += platforms->angular_weight * platforms->angular_motions[i].height / 2048;
	
	mimimi_platforms_propagate(platforms);
	
	if (platforms->momentum.x < platforms->min)
	{
		platforms->momentum.x = platforms->min;
		platforms->momentum.dx = 0;
	}
	if (platforms->momentum.x > platforms->max)
	{
		platforms->momentum.x = platforms->max;
		platforms->momentum.dx = 0;
	}
	
	platforms->momentum.dx *= 31;
	platforms->momentum.dx /= 32;
	platforms->momentum.x += platforms->momentum.dx / 256;
	
	mimimi_platforms_apply(platforms);
	
	for (i = 0 ; i < platforms->linear_platform_count ; i++)
		mimimi_positioned_physics_tick(&platforms->linear_platforms[i].physics, platforms->linear_platforms[i].ground);
	for (i = 0 ; i < platforms->angular_platform_count ; i++)
		mimimi_positioned_physics_tick(&platforms->angular_platforms[i].physics, platforms->angular_platforms[i].ground);
	
	diff = platforms->balance - platforms->momentum.x;
	platforms->momentum.dx += platforms->strength * diff / 256;
}

void mimimi_platform_set_display_tick(struct mimimi_platform_set *platforms, struct mimimi_engine *engine, struct mimimi_position *camera)
{
	int i;
	for (i = 0 ; i < platforms->linear_platform_count ; i++)
		mimimi_platform_display_tick(platforms->linear_platforms + i, camera, 0, 0, engine);
	for (i = 0 ; i < platforms->angular_platform_count ; i++)
		mimimi_platform_display_tick(platforms->angular_platforms + i, camera, 0, 0, engine);
}
