/* license: AGPLv3 or later */
/* copyright 2024 zamfofex */

#include <mimimi.h>

void mimimi_linear_propagate(struct mimimi_momentum *momentum, int value, int x0, int scale)
{
	momentum->dx += ((value - x0) * 256 * 256 / scale - momentum->x) * 256;
}

void mimimi_linear_apply(struct mimimi_momentum *momentum, int *value, int x0, int scale)
{
	*value = momentum->x * scale / 256 + x0;
}

void mimimi_angular_propagate(struct mimimi_momentum *momentum, struct mimimi_position *position, int x0, int y0, int a0, int scale)
{
	int a, x, y, dx;
	
	x = position->x - x0;
	y = position->y - y0;
	a = mimimi_angle(x, y);
	
	dx = (a * 4 - a0 * 256) * 256 - momentum->x * scale;
	dx = mimimi_mod(dx + 0x800000, 0x1000000) - 0x800000;
	momentum->dx += dx / scale;
}

void mimimi_angular_apply(struct mimimi_momentum *momentum, struct mimimi_position *position, int x0, int y0, int a0, int scale, int length)
{
	int a, x, y;
	
	a = mimimi_mod(momentum->x * scale / 256 / 32 + a0 * 8, 2048);
	
	x = mimimi_sine[a] * length / 128;
	y = mimimi_cosine[a] * length / 128;
	
	position->x = x + x0;
	position->y = y + y0;
}

void mimimi_linear_motion_propagate(struct mimimi_linear_motion *motion, struct mimimi_momentum *momentum)
{
	mimimi_linear_propagate(momentum, *motion->value, motion->origin, motion->scale);
}

void mimimi_linear_motion_apply(struct mimimi_linear_motion *motion, struct mimimi_momentum *momentum)
{
	mimimi_linear_apply(momentum, motion->value, motion->origin, motion->scale);
}

void mimimi_linear_motion(struct mimimi_linear_motion *motion, int *value, int origin, int scale)
{
	motion->value = value;
	motion->origin = origin;
	motion->scale = scale;
}

void mimimi_angular_motion_propagate(struct mimimi_angular_motion *motion, struct mimimi_momentum *momentum)
{
	mimimi_angular_propagate(momentum, motion->value, motion->origin.x, motion->origin.y, motion->angle, motion->scale);
}

void mimimi_angular_motion_apply(struct mimimi_angular_motion *motion, struct mimimi_momentum *momentum)
{
	mimimi_angular_apply(momentum, motion->value, motion->origin.x, motion->origin.y, motion->angle, motion->scale, motion->height);
}

void mimimi_angular_motion(struct mimimi_angular_motion *motion, struct mimimi_position *value, int x0, int y0, int a0, int scale, int height)
{
	motion->value = value;
	motion->origin.x = x0;
	motion->origin.y = y0;
	motion->angle = a0;
	motion->scale = scale;
	motion->height = height;
}
